from setuptools import setup, find_packages

setup(
    name='xianb',
    version='1.1.4',
    author='jxjiang',
    author_email='723137901@qq.com',
    description='',
    python_requires='>=3.8',
    install_requires=[],
    packages=find_packages()
)
